USE AllNasdaqTickerPricesfrom2014into2017
GO

-- create fresh copy of ##symbol table
-- with symbol and symbol_number columns
-- for all historical prices and volumes
-- in Results_with_extracted_casted_values

begin try
drop table ##symbol
end try
begin catch
print '##symbol not available to drop'
end catch

select 
[symbol]
,row_number() over (order by symbol) AS symbol_number
into ##symbol
from
(
select distinct symbol
from  Results_with_extracted_casted_values
) for_distinct_symbols
order by symbol


-- create a fresh copy of the ewma_10_30_50_200 table
-- to preserve exponential moving averages
begin try
drop table [ewma_10_30_50_200]
end try
begin catch
print '[ewma_10_30_50_200] not available to drop'
end catch 

create table [dbo].[ewma_10_30_50_200](
	[symbol] [varchar](10) NULL,
	[date] [date] NULL,
	[close] [money] NULL,
	[ewma_10] [money] NULL,
	[ewma_30] [money] NULL,
	[ewma_50] [money] NULL,
	[ewma_200] [money] NULL
)



-- declare local variables to help
-- loop through stock symbols to populate ewma_10_30_50_200
declare @maxPK int;Select @maxPK = MAX(symbol_number) From ##symbol
declare @pk int;Set @pk = 1
declare @symbol varchar(5)


-- start while loop for successive @symbol values
while @pk <= @maxPK
begin



-- initially populate #temp_for_ewma for ewma calculations

-- select @symbol value for current pass through the while loop
-- @ewma_first is the seed
set @symbol = (select [symbol] from ##symbol where symbol_number =  @pk)
declare @ewma_first money = (select top 1 [close] from Results_with_extracted_casted_values where symbol = @symbol order by [date])


-- create base temporary table (#temp_for_ewma) for ewma calculations
-- with NULL values for first row and seed values for second row
-- third and subsequent rows are updated by recursive expressions
-- when passing through dates for current @symbol value

begin try
drop table #temp_for_ewma
end try
begin catch
print '#temp_for_ewma not available to drop'
end catch

-- assign ewma seed values
select 
 [date]
,[symbol]
,[close]

,row_number() OVER (ORDER BY [Date]) [row_number]
,@ewma_first ewma_10
,@ewma_first ewma_30
,@ewma_first ewma_50
,@ewma_first ewma_200

into #temp_for_ewma
from Results_with_extracted_casted_values
where symbol = @symbol
--order by row_number

-- NULL ewma values for first period
update #temp_for_ewma
set
	ewma_10 = NULL
   ,ewma_30 = NULL	
   ,ewma_50 = NULL	
   ,ewma_200 = NULL	
where row_number = 1



-- calculate ewma_10, ewma_30, ewma_50, and ewma_200
-- @ew_10 is the exponential smoothing constant for 10-period
-- @ew_30 is the exponential smoothing constant for 30-period
-- @ew_50 is the exponential smoothing constant for 50-period
-- @ew_200 is the exponential smoothing constant for 200-period
-- seed is close from 1st period; it is used as ewma for 2nd period 
-- start recursive calculations with the 3rd period value

declare @max_row_number int = (select max(row_number) from #temp_for_ewma)
declare @current_row_number int = 3
declare 
 @ew_10 real = 2.0/(10.0 + 1), @today_ewma_10 real
,@ew_30 real = 2.0/(30.0 + 1), @today_ewma_30 real
,@ew_50 real = 2.0/(50.0 + 1), @today_ewma_50 real
,@ew_200 real = 2.0/(200.0 + 1), @today_ewma_200 real


-- use while statement to loop through successive date rows 
-- for current @symbol value
while @current_row_number <= @max_row_number
begin

	set @today_ewma_10 =
	(
	-- compute ewma_10
	select 
	top 1
	([close] * @ew_10) + (lag(ewma_10,1) over (order by [date]) * (1 - @ew_10)) ewma_10_today

	from #temp_for_ewma
	where row_number >=  @current_row_number -1 and row_number <=  @current_row_number
	order by row_number desc
	)

	set @today_ewma_30 =
	(
	-- compute ewma_30
	select 
	top 1
	([close] * @ew_30) + (lag(ewma_30,1) over (order by [date]) * (1 - @ew_30)) ewma_30_today

	from #temp_for_ewma
	where row_number >=  @current_row_number - 1 and row_number <=  @current_row_number
	order by row_number desc
	)

	set @today_ewma_50 =
	(
	-- compute ewma_50
	select 
	top 1
	([close] * @ew_50) + (lag(ewma_50,1) over (order by [date]) * (1 - @ew_50)) ewma_50_today

	from #temp_for_ewma
	where row_number >=  @current_row_number - 1 and row_number <=  @current_row_number
	order by row_number desc
	)

	set @today_ewma_200 =
	(
	-- compute ewma_200
	select 
	top 1
	([close] * @ew_200) + (lag(ewma_200,1) over (order by [date]) * (1 - @ew_200)) ewma_200_today

	from #temp_for_ewma
	where row_number >=  @current_row_number - 1 and row_number <=  @current_row_number
	order by row_number desc
	)


	update #temp_for_ewma
	set 
		ewma_10 = @today_ewma_10
	   ,ewma_30 = @today_ewma_30	
	   ,ewma_50 = @today_ewma_50	
	   ,ewma_200 = @today_ewma_200	
	where row_number =  @current_row_number

	set @current_row_number =  @current_row_number + 1

end

-- insert #temp_for_ewma for current value  
-- of @symbol into ewma_10_30_50_200
insert into ewma_10_30_50_200
select
symbol
,[date]
,[close]
,ewma_10
,ewma_30
,ewma_50
,ewma_200
 from #temp_for_ewma


-- update @pk value for next set of 
-- @symbol exponential moving averages 
    Select @pk = @pk + 1

end



