
USE AllNasdaqTickerPricesfrom2014into2017
GO

--/*

-- use this segment of the script to set up for calculating
-- ewma_10 and ewma_30 for the AAAP ticker symbol

-- initially populate #temp_for_ewma for ewma calculations
-- @ewma_first is the seed
declare @symbol varchar(5) = 'AAAP'
declare @ewma_first money = (select top 1 [close] from Results_with_extracted_casted_values where symbol = @symbol order by [date])


-- create base table for ewma calculations

begin try
drop table #temp_for_ewma
end try
begin catch
print '#temp_for_ewma not available to drop'
end catch

-- ewma seed run
select 
 [date]
,[symbol]
,[close]

,row_number() OVER (ORDER BY [Date]) [row_number]
,@ewma_first ewma_10
,@ewma_first ewma_30
into #temp_for_ewma
from Results_with_extracted_casted_values
where symbol = @symbol
order by row_number

-- NULL ewma values for first period
update #temp_for_ewma
set
	ewma_10 = NULL
   ,ewma_30 = NULL	
where row_number = 1



-- calculate ewma_10 and ewma_30
-- @ew_10 is the exponential weight for 10-period
-- @ew_30 is the exponential weight for 30-period
-- start calculations with the 3rd period value
-- seed is close from 1st period; it is used as ewma for 2nd period 

declare @max_row_number int = (select max(row_number) from #temp_for_ewma)
declare @current_row_number int = 3
declare @ew_10 real = 2.0/(10.0 + 1), @today_ewma_10 real
declare @ew_30 real = 2.0/(30.0 + 1), @today_ewma_30 real

while @current_row_number <= @max_row_number
begin

	set @today_ewma_10 =
	(
	-- compute ewma_10 for period 3
	select 
	top 1
	([close] * @ew_10) + (lag(ewma_10,1) over (order by [date]) * (1 - @ew_10)) ewma_10_today

	from #temp_for_ewma
	where row_number >=  @current_row_number -1 and row_number <=  @current_row_number
	order by row_number desc
	)

	set @today_ewma_30 =
	(
	-- compute ewma_30 for period 3
	select 
	top 1
	([close] * @ew_30) + (lag(ewma_30,1) over (order by [date]) * (1 - @ew_30)) ewma_30_today

	from #temp_for_ewma
	where row_number >=  @current_row_number - 1 and row_number <=  @current_row_number
	order by row_number desc
	)


	update #temp_for_ewma
	set 
		ewma_10 = @today_ewma_10
	   ,ewma_30 = @today_ewma_30	
	where row_number =  @current_row_number

	set @current_row_number =  @current_row_number + 1

end

-- display the result set with the calculated values
select *
from #temp_for_ewma
where row_number < = @max_row_number
order by row_number


--*/



--/*
-- 10-period and 30-period squared deviation
-- comparisons of arthimetic (mav) vs. exponential 
-- moving averages

select
from_ewma_demo.symbol
,from_ewma_demo.[date]
,from_mav_10_30_50_200.[close]
,from_mav_10_30_50_200.mav_10
,from_mav_10_30_50_200.mav_30
,from_ewma_demo.ewma_10
,from_ewma_demo.ewma_30
,([close]-mav_10) * ([close]-mav_10) mav_10_squared_deviation
,([close]-ewma_10) * ([close]-ewma_10) ewma_10_squared_deviation
,([close]-mav_30) * ([close]-mav_30) mav_30_squared_deviation
,([close]-ewma_30) * ([close]-ewma_30) ewma_30_squared_deviation

from
(
SELECT TOP 1000 [symbol]
      ,[date]
      ,[open]
      ,[close]
      ,[mav_10]
      ,[mav_30]

FROM [AllNasdaqTickerPricesfrom2014into2017].[dbo].[mav_10_30_50_200]
where symbol = @symbol
) from_mav_10_30_50_200

left join

(
select 
[date]
,symbol
,ewma_10
,ewma_30
from #temp_for_ewma
) from_ewma_demo

on from_mav_10_30_50_200.[date] = from_ewma_demo.[date]

order by from_mav_10_30_50_200.[date] desc


--*/
