
USE AllNasdaqTickerPricesfrom2014into2017
GO


-- initially populate #temp_for_ewma for ewma calculations
-- @ewma_first is the seed
declare @ewma_first real = (select top 1 [tavg] from daily_temps_from_1_Jan_14_thru_8_Nov_17_casted /*(where symbol = @symbol */order by [date])
declare @min_date date = (select top 1 [date] from daily_temps_from_1_Jan_14_thru_8_Nov_17_casted /*(where symbol = @symbol */order by [date])

--select @min_date date, @ewma_first


-- create base table for ewma calculations
begin try
drop table #temp_for_ewma
end try
begin catch
print '#temp_for_ewma not available to drop'
end catch

-- ewma seed run
select 
 [date]
,tavg
,row_number() over (order by [date]) [row_number]
,@ewma_first ewma_10
,@ewma_first ewma_30
into #temp_for_ewma
from [daily_temps_from_1_Jan_14_thru_8_Nov_17_casted]
--order by [date]

-- NULL ewma values for first period
update #temp_for_ewma
set
	ewma_10 = NULL
   ,ewma_30 = NULL	
where [date] = @min_date



-- calculate ewma_10 and ewma_30
-- @ew_10 is the exponential weight for 10-period
-- @ew_30 is the exponential weight for 30-period
-- start calculations with the 3rd period value
-- seed is tavg from 1st period; it is used as ewma for 2nd period 

declare @max_row_number int = (select max(row_number) from #temp_for_ewma)
declare @current_row_number int = 3
declare @ew_10 real = 2.0/(10.0 + 1), @today_ewma_10 real
declare @ew_30 real = 2.0/(30.0 + 1), @today_ewma_30 real

while @current_row_number <= @max_row_number
begin

	set @today_ewma_10 =
	(
	-- compute ewma_10 for period 3
	select 
	top 1
	([tavg] * @ew_10) + (lag(ewma_10,1) over (order by [date]) * (1 - @ew_10)) ewma_10_today

	from #temp_for_ewma
	where row_number >=  @current_row_number -1 and row_number <=  @current_row_number
	order by row_number desc
	)

	set @today_ewma_30 =
	(
	-- compute ewma_30 for period 3
	select 
	top 1
	([tavg] * @ew_30) + (lag(ewma_30,1) over (order by [date]) * (1 - @ew_30)) ewma_30_today

	from #temp_for_ewma
	where row_number >=  @current_row_number - 1 and row_number <=  @current_row_number
	order by row_number desc
	)


	update #temp_for_ewma
	set 
		ewma_10 = @today_ewma_10
	   ,ewma_30 = @today_ewma_30	
	where row_number =  @current_row_number

	set @current_row_number =  @current_row_number + 1

end

select *
from #temp_for_ewma
order by row_number

